import gymnasium as gym
import panda_gym
from stable_baselines3 import A2C
from stable_baselines3.common.monitor import Monitor
from stable_baselines3.common.vec_env import DummyVecEnv
import wandb
from wandb.integration.sb3 import WandbCallback
from huggingface_hub import login
from huggingface_sb3 import push_to_hub


if __name__ == "__main__":
    config = {
        "policy_type": "MultiInputPolicy",
        "total_timesteps": 500000,
        "env_name": "PandaReachJointsDense-v3",
    }

    wandb.login(key='xxx')

    run = wandb.init(
        project="pandareach",
        config=config,
        sync_tensorboard=True,
        monitor_gym=True,
        save_code=True,
    )

    env = gym.make(config["env_name"])
    model = A2C(config["policy_type"], env, verbose=1, tensorboard_log=f"runs/{run.id}")
    model.learn(
        total_timesteps=config["total_timesteps"],
        callback=WandbCallback(
        )
    )

    run.finish()

    model.save("a2c_pandareach")

    login(token="xxx")
    push_to_hub(
        repo_id="Thomstr/A2C_PandaReach",
        filename="a2c_pandareach.zip",
        commit_message="Added A2C model for PandaReach with Stable Baselines3",
    )