import torch
import gym
import torch.nn as nn
import torch.nn.functional as F
from torch.distributions import Categorical


# Define the CNN architecture
class Net(nn.Module):
    def __init__(self):
        super(Net, self).__init__()
        self.fc1 = nn.Linear(4, 128) # input : size 4 environment observation
        self.fc2 = nn.Linear(128, 2)  # output : 2 possible output (left or right)
        self.dropout = nn.Dropout(0.25)
        self.softmax = nn.Softmax(dim=0)

    def forward(self, x):
        x = F.relu(self.fc1(x))
        x = self.dropout(x)
        x = self.fc2(x)
        x = self.softmax(x)
        return x


def select_action(model, state):
    # Get the new probabilities of action
    prob = model(torch.tensor(state))
    # Sample the action based on the probabilities and store its probability in the buffer
    m = Categorical(prob)  # create a discreet distribution
    action = m.sample()  # choose a random action following the probabilities
    return action


def evaluate(model, env, episodes=100):
    successes = 0
    for ep in range(episodes):
        print(f"{ep}/{episodes}")
        state, _ = env.reset()
        total_reward = 0
        done = False
        truncated = False
        step = 0
        while not (done or truncated):
            action = select_action(model, state)
            state, reward, done, truncated, _ = env.step(action.item())
            total_reward += reward
            step += 1
        if truncated:
            successes += 1
    return successes / episodes


if __name__ == "__main__":
    env = gym.make("CartPole-v1", render_mode=None)
    model = Net()
    model.load_state_dict(torch.load("reinforce_cartpole_best.pth"))
    model.eval()
    success_rate = evaluate(model, env)
    print(f"Success Rate: {success_rate * 100:.2f}%")